#  This is an AWK script for use with Rob Duff's AWK for the PC.
#
#  This script is for processing an ASCII file of mail messages
#  which were archieved using QMail. The output is a header and a data file
#  as per the QWK packet format.
#  
#  The purpose of this allow you to switch from using QM as your mail reader
#  to the vastly superior PMDBM - Personal Message Database Manager by Les
#  Gyurky.  Even PMDBM is not perfect.  It cannot accept messages previously
#  archived from QM.  Which is where this scrip comes in.
#  
#  The AWK program, using this script can process all Archive files in a 
#  directory and create the QWK file MESSAGES.DAT in QWK packet format.
#  
#	This version does not create CONTROL.DAT of the QWK packet
#	To import into PMDBM, create a .QWK packet (using PKZIP) that contains
#	the MESSAGES.DAT file created here and any version of CONTROL.DAT  (from
#	a downloaded .QWK file.  CONTROL.DAT does not need to match MESSAGES.DAT
#	Move  the newly created .QWK file to your QWK directory (wherever PMDBM
#	expects the file to be and import/read.
#
#	There is one know bug in this AWK script (actually the bug is in AWK.EXE).
#	The .QWK file requires the conference no in binary.  thus we use the
#	statement  printf "%c%c", n1, n2 to write out that binary number, where
#	n1 & n2 were calculated as the low byte and the high byte of confno, 
#	the conference no.
#	Unfortunately the AWK.EXE program used fails if n1 or n2 = 0 (presumably
#	because 0x00 in C is treated as end of string, resulting in fewer bytes 
#	being written than expected, and a corrupted MESSAGES.DAT file.
#
#	Whenever n1 or n2 is 0, the message is assigned to conf 8192 and the
#	conf name set to "Qmail Archive").  This keeps the MESSAGES.DAT file of
#	the proper format.
#
#	If there are message number duplicates, PMDBM will discard these duplicates
#	This may happen since messages archived from different conferences
#	all end up in 8192.
#
#	By Frans van Duinen
#	12 Apr 92
#
#  Usage:
#
#  AWK QWK.AWK  *.TXT 
#
#	This script always creates 2 files, MESSAGES.DAT and CONTROL.DAT
#
#  The input files are assumed to have the following format:
#   	a line containing only the character ""  (start of msg line)
#  	    seven lines of message header, always having the same layout (fixed 
#  	   		format) first line BBS: ....., last line 
#  		zero or more lines of message text
#   	a line containing only the character ""  (end of msg line)
#	These 4 sections are repeated for each additional message in the file.
#
#	An example is shown below:
#
#  
#   BBS: CRS Premium Bulletin Board - ATI ETC  (416) 629-7000
#  Date: 01-11-92 (22:34)              Number: 3488
#    To: MIKE LIPSCOMB                 Refer#: 3481                 (Msg #9 of 35)
#  From: KARL STEER                      Read: NO
#  Subj: CLIPPER .DBF TO .WKS          Status: PUBLIC MSG
#  Conf: R-CLIPPER (493)            Direction: FORWARD
#  
#  ML>Does anyone know of a Clipper UDF that will convert an Xbase format
#  ...
#    PMDBM V1.7 #19  I am not young enough to know everything.
#  
#  
#   BBS: CRS Premium Bulletin Board - ATI ETC  (416) 629-7000
#  Date: 01-11-92 (22:34)              Number: 3489
#  ...
#
#  
#  The following diescription of the QWK file format are from Mark May's
#  file QWKSPC12.ZIP:
#  
#                          QWK mail format information
#  
#                   By Mark May - Mythical Kingdom Software
#  
#          While writing a QWK utility, I found that there was no
#          one place where all the necessary info on the QWK format
#          was available.  Thru messages in several network echo
#          conferences (Thanks to Tim Farley, Mark Goodwin, Randy
#          Blackmond, Mark Herring, Patrick Lee, Greg Hewgill, and
#          others), I gradually collected the information enclosed
#          in this archive.
#  
#          To save others the time collecting this information, I've
#          put together this set of files by editing the information
#          from the various sources so that it is conveniently
#          available.
#  
#          Tim Farley is working on a more rigorous specification and
#          I'm sure all of us would appreciate any help you can give
#          him in this.   If you find any errors or additional
#          information, I'd also appreciate hearing about it so that
#          I can further update this set of files.
#  
#          I am working on an Object-Oriented set of TurboPascal routines
#          that I hope to release to make working with QWK/REP files
#          easier.
#  
#          Note that this packet of information does not document net-status.
#          QWK packets that are marked as having net-status may be imported
#          by one of the NET utils (QNet, RNet, MKNet, etc) into a Bbs.
#          Because normal users of a QWK door should not be able to hack a
#          packet to give it net-status (and import it into their system),
#          that info has been left out of this set of documents.  You may
#          contact me (or one of the other QWK authors) directly for this
#          info.
#  
#  
#          You may contact me at:
#  
#          Fido - 1:110/290
#          RIME - ->MYTHKING
#          Normal Bbs 513-237-7737 (at 9600 HST/V32)
#  
#          You may also contact me in the P/Net (programmers network)
#          Bbs/Utils developers conference.
#  
#  
#          Changes:
#  
#          v1.2 01/01/92 Added info on Personal.Ndx
#                        Cleaned up code appearance on BasicReal conversions
#                        Added note about missing net-status info
#  
#          v1.1 08/17/91 Added information on Door.Id
#                        Added info on conferences above 255
#                        Added info on message has tagline field
#                        Corrected info on message status values
#  
#          v1.0 05/28/91 Initial release
#
#                Format of the exported messages in Messages.Dat
#
#
#           This file contains records with a length of 128 bytes.  There
#           are 3 types of these records: (1) Packet Header, (2) Message
#           Header, and (3) Message Text.  All unused fields in the
#           records are normally filled with spaces, although you will
#           sometimes find the final Message text record will be filled
#           with nulls (#0) after the last text.
#
#
#                                Packet Header
#
#           Packet Header - is always the first record in the file and
#           only occurs once.  It contains only normal ascii text
#           (limitted to at most 128 characters) and should always
#           start with "Produced by ".  The remaining text normally
#           includes a product name and copyright message.
#
#
#                                Message Header
#
#           A message header immediately preceeds zero or more message
#           text records.  Each Message header has the following
#           format:
#
#           Start
#            Pos  Length              Description
#          ------ ------ -----------------------------------------
#             1      1   Message status flag (see below)
#             2      7   Message number (ascii left justified)
#             9      8   Date (MM-DD-YY)
#            17      5   Time (HH:MM)
#            22     25   To (left justified space filled - uppercase)
#                        (Be sure to check the to field to spot
#                        configuration type messages)
#            47     25   From (left justified space filled - uppercase)
#            72     25   Subject (left justified space filled - uppercase)
#                        (a subject starting "NE:" should not be
#                        echoed into a network)
#            97     12   Password (not really used leave blank)
#           109      8   Message reference number (ascii left justified)
#           117      6   Number of blocks (ascii left justified - number
#                        of 128 byte blocks including 1 for the message
#                        header)
#           123      1   Message active( or #225 = Active,
#                         or #226 = Inactive)
#           124      2   Conference number (Binary Word) (Lo in 124, Hi
#	                     in 125).  Note that some older programs only
#                        supported conferences 0 thru 255 and left byte
#                        125 as a "blank" unused field.  I recommend
#                        checking the word value and if it is over 8191
#                        to only use the lower (0 thru 255) part of it.
#                        I've also seen notes that this field is really
#                        a 4 byte LongInt field, although with the above
#                        limitation (and 8000 conference seeming sufficient
#                        for a while), I documented it this way.
#
#           128      1   Indicates whether the message has a Network
#                        tagline or not.  A value of "*" indicates that
#                        a network tagline is present, a value of " "
#                        indicates there isn't.  Messages sent to readers
#                        (non-net-status) generally leave this as always
#                        blank since only net utils need this info.
#
#           Message Status Flag has the following possibilities:
#
#           '`'  Comment to sysop, read      '~'  Comment to sysop, unread
#           '-'  Public, read                ' '  Public, unread
#           '*'  Private, read               '+'  Private, unread
#           '^'  Password protected, read    '%'  Password protected, unread
#           '#'  Group password, read        '!'  Group password, unread
#           '$'  Group password to all
#
#
#
#                               Message Text Records
#
#           The message text records immediately follow the the message
#           header.  They contain straight ascii text (lines are
#           normally limitted to 72 chars/line although you may see
#           longer lines).  Each line is followed by a "" or #227
#           character to mark the end of the line (in place of the normal
#           CR/LF that would exist in a straight text file).  The text
#           continues consecutavly and text lines do continue across
#           block boundaries.  Some systems or readers may have problems
#           with messages longer than 99 lines or 199 lines, although more
#           recently this no longer seems to be a limit.  The last block
#           should be padded with blanks to fill the block, although on
#           input you may find it padded with nulls (#0).
#
#
#	
#	
#	                        Format of Control.Dat file
#	
#	      Control.Dat is a normal CR/LF style text file with the
#	      following lines in exactly the order shown.
#	
#	BBS name          					# PMDBM checks these 2/3 lines
#	BBS City, BBS State
#	BBS phone number                     {AAA-EEE-NNNN}
#	Sysop name,Sysop                     {Sysop name in upper case followed
#	                                      by the literal ',Sysop'}
#	Serial number ,BBS ID                {Serial Number, UpperCase BBSID}
#	Date of mail packet, time of packet  {MM-DD-YYYY,HH:MM:SS}
#	Caller's name                        {Uppercase}
#	blank
#	0
#	0
#	Number of conferences (in additon to conference 0 which is main)
#	                      (ie use number of conferences minus one)
#	Conference number
#	Name of Conference    (should be limited to maximum length of 12 characters)
#	...
#	...
#	...
#	...
#	...
#	Conference number
#	Name of Conference
#	name of BBS welcome file
#	name of NEWS file
#	name of BBS goodbye file
#	
#	Other optional data may occur after the Goodbye file line, but the
#	trend is to omit this data.  If included it is as follows:
#	
#	0                                    {Unknown}
#	Screen Length
#	USER NAME                            {Upper case}
#	FirstName                            {Proper case}
#	CITY, ST                             {Upper case}
#	Data Phone                           {AAA EEE-NNNN}
#	Voice Phone                          {AAA EEE-NNNN}
#	Security Level
#	Expiration Date                      {MM-DD-YY}
#	Last On Date                         {MM-DD-YY}
#	Last On Time                         {HH:MM}
#	Number of calls
#	0                                    {Unknown}
#	DownLoaded Bytes
#	DownLoaded Count
#	Uploaded Bytes
#	Uploaded Count
#	Time Limit Per Day                   {Minutes}
#	Time Remaining                       {Minutes}
#	Time Used Today                      {Minutes}
#	DownLoad Limit/Day                   {Kilobytes}
#	DownLoad Bytes Remaining Today       {Bytes}
#	DownLoaded Today                     {Bytes}
#	Current Time                         {HH:MM}
#	Current Date                         {MM-DD-YY}
#	System Tag Line
#	
#
#
BEGIN { mflag = 0;										# Header line no
	msgfn = "MESSAGES.DAT";  ctlfn = "CONTROL.DAT";		# names of files created
	bbs = "";   date = ""; msgno = "";    msgto = "" ;  # Hdr fields
	refno = ""; from = ""; readstus = ""; subj = "";	# ...
	stus = "";  conf = ""; confno = 0;    dir = "";     # ...
	msgstus = "-";										# Public, read
	debug = 0;											# 0, 1 or 2 
  	msgcount = 0;										# # of msgs
	txtlen = 0;											# total text length
	lines = "";											# array of lines
	confnames[ 0] = "MAIN";
	printf "%-128s", "Produced by Frans van Duinen's Archive to QWK AWK" > msgfn
														# QWK Packet header
}

(debug > 1)  { print}	# debug - list input

#	Check for  as line before message
{if ($0 == "") {mflag = 1; msgcount++; nolines = 0;		# Start of msg
										txtlen = 0;
										lines = "";
#				print "Message start marker"		# debug - show progress
				}

#	Process BBS Name (Hdr line 1)
 else {if (mflag == 1) { mflag++; BBS = $0					# not used
#				print "Header line 1 ", $0 		# debug - show progress
					    }

#	Get Date, Time  & msg no
 else {if (mflag == 2) { mflag++; date = $2;
								  time = substr( $3, 2, 5) ; # must strip ( )
								  msgno = $5;
#				print "Header line 2 ", $0		# debug - show progress
					    }

#	To & Ref no
 else {if (mflag == 3) { mflag++; msgto = substr( $0, 7, 25) ; # name
								  refno = substr( $0, 45, 8) ;
#				print "Header line 3 ", $0 		# debug - show progress
					    }

#	From & Read stus
 else {if (mflag == 4) { mflag++; from = substr( $0, 7, 25) ;
								  readstus = $NF;				# last field
#				print "Header line 4 ", $0 		# debug - show progress
						}

#	Subj & status
#	Status text may be 2 - 4 fields, subject any number  - extract # of chars
 else {if (mflag == 5) { mflag++; subj = substr( $0, 7, 25) ;
								  stus = substr( $0, 45) ;
#				 print "Header line 5 ", $0		# debug - show progress
						}

#	Conf, Conf no & direction
 else {if (mflag == 6) { mflag++; conf = $2; 		# Assume one word name
								  confno = substr( $3, 2) +0;	# to no to strip ( )
		 	  					  dir = $5 ;
#				print "Header line 6 ", $0 		# debug - show progress
						}

 else {if (mflag == 7 && $0 !~ /~/ ) { mflag++; 	# only if line =  ...
#				print "Header line 7 ", $0 		# debug - show progress
						}

#	End of msg marker 
 else {if ($0 == "") {mflag = 0;
		noblocks = (txtlen+127)/ 128			# no of text blocks (not lines)
#		print "Message end marker";				# debug - show progress
#  		print "MsgStus ", msgstus ;# Debug
#		print "Msgno   ", msgno;
#		print "Date    ", date;
#		print "Time    ", time;
#		print "Msgto   ", msgto;
#		print "From    ", from;
#		print "Read sts", readstus;
#		print "Subject ", subj;
#		print "Passwd  ", "            ";
#		print "Refno   ", refno;
#		print "No blks ", noblocks+1; #
#		print "Msg stus", ""; 				# Message active
#		print "Conf    ", conf,  "   Conf no", confno
#		print "Tagflag ", " "; 				# Tag line flag
#		for ( i=1; i <=nolines; i++) { print i, lines[ i]};
#		print "Tot length", txtlen, "to pad", 128 - (txtlen % 128) 
		# now pad to 128 multiple				# END DEBUG

		if (confno < 1  || confno > 8192) { confno = 8192 };
		n1 = confno % 256
		confno = int( confno / 256)
		n2 = confno % 256
		if ( n2 == 0 || n1 == 0)  { n2 = 32; n1 = 32; 
									conf = "Qmail Archive"}		
				# Bug in AWK, cant handle printf "%c", 0, so any
				# that result in n2 = 0 or n1 = 0 are assigned to 8192
		confno = n1 + 256 * n2	# Conf no we're actually using

				printf "%1c",  msgstus >  msgfn  ;		# 1, 1
				printf "%-7d", msgno   >  msgfn  ;
				printf "%8s",  date    >  msgfn  ;
				printf "%5s",  time    >  msgfn  ;
				printf "%25s", msgto   >  msgfn  ;
				printf "%25s", from    >  msgfn  ;
				printf "%25s", subj    >  msgfn  ;
				printf "%12s", "            "   >  msgfn  ;
				printf "%8s",  refno   >  msgfn  ;			
				printf "%-6d", noblocks+1   >  msgfn  ; #
				printf "%1c",  ""     >  msgfn  ; 		# Message active
				printf "%c%c", n1, n2  >  msgfn ;		# Conf no in binary
				printf "%2c",  "  "	   >  msgfn ; 		# filler 
				printf "%1c",  "*" 	   >  msgfn ; 		# Tag line flag
				for ( i=1; i <=nolines; i++) { 
						  	 printf "%s%c", lines[ i], ""   >  msgfn  };
				nolines = 128 - (txtlen % 128) 		 	# pad to 128 multiple
				for ( i=1; i <=nolines; i++) {
							 printf "%c", " "   >  msgfn  };	
												# write pad char
				if ( confno in confnames) 
#					 { print "in array", conf; }		# debug
				else { confnames[ confno] = conf; 
#					   print "added to array", conf; 	# debug
					 }
#		exit;									# debug - exit after 1 msg
		}

#	Process message text
 else { txtlen = txtlen + length( $0) +1;		# message length incl 
		nolines++;
		lines[ nolines] = $0;
#		print "Message text '", $0, "'"			# debug - list input
	   }
 }}}}}}}}}

	# Change the next three print statements to match the information 
	# in the CONTROL.DAT file as downloaded from your BBS.
	# PMDBM uses this to verify that the .QWK packet is for this
	# particular message database (one such database per BBS)
END { print "CRS Premium Bulletin Board - ATI ETC" >  ctlfn  ;
	  print "" >  ctlfn ;
	  print"(416) 629-7000" >  ctlfn  ;

	  print"CRSSYSOP JUD, Sysop" >  ctlfn  ;
	  print"104 ,CRS" >  ctlfn  ;
	  print"04-11-1992,11:20:05" >  ctlfn  ;
	  print"your user id " >  ctlfn  ;
	  print"MENU" >  ctlfn  ;
	  print"0" >  ctlfn  ;
	  print"0" >  ctlfn  ;
	  i = 0; for ( confno in confnames) { i++}
	  print i-1 >  ctlfn  ;					 # no of confs less 1
	  for ( x in confnames) { print x  >  ctlfn  ;
							 print confnames[ x]  >  ctlfn  };
	  print"WELCOME" >  ctlfn  ;
	  print"BBSNEWS" >  ctlfn  ;
	  print"GOODBYE" >  ctlfn  ;
	  print"0"  >  ctlfn  ;
	print "\n", msgcount, "messages"; 	# debug
	print "in", i-1, "conferences"; 	# debug
	}

